import xmlrpc.client

from django.conf import settings

ADDRESS_FIELDS = [
    "id",
    "name",
    "street",
    "street2",
    "city",
    "zip",
    "state_id",
    "country_id",
    "email",
    "phone",
    "vat",
    "company_type",
]


def odoo_request(model, method, **kwargs):
    url = settings.ODOO["URL"]
    db = settings.ODOO["DB"]
    username = settings.ODOO["USERNAME"]
    password = settings.ODOO["PASSWORD"]

    try:
        common = xmlrpc.client.ServerProxy(f"{url}/xmlrpc/2/common")
        uid = common.authenticate(db, username, password, {})

        if not uid:
            raise Exception("Authentication failed with Odoo.")

        models = xmlrpc.client.ServerProxy(f"{url}/xmlrpc/2/object")

        # Prepare arguments for execute_kw
        # Odoo's execute_kw expects: db, uid, password, model, method, args_list, kwargs_dict
        # For 'search_read', args_list typically contains [domain, fields]
        # and kwargs_dict contains {'limit': ..., 'offset': ..., 'order': ...}

        args_list = []
        kwargs_dict = {}

        if method == "search_read":
            # Extract domain and fields for positional arguments if present
            domain = kwargs.pop("domain", [])
            fields = kwargs.pop("fields", [])
            args_list = [domain, fields]
            # Remaining kwargs are passed as the options dictionary
            kwargs_dict = kwargs
        else:
            # For other methods, we might need a more generic way or specific handling.
            # For now, assume kwargs can be passed directly if method is not 'search_read',
            # or that they are passed as a list of arguments.
            # This part might need refinement based on other Odoo methods used.
            # A common pattern is to pass a list of IDs as the first arg for methods like 'read', 'write'.
            # If 'args' is explicitly passed in kwargs, use it.
            if "args" in kwargs:
                args_list = kwargs.pop("args")
            # Remaining kwargs are passed as the options dictionary
            kwargs_dict = kwargs

        breakpoint()
        result = models.execute_kw(
            db, uid, password, model, method, args_list, kwargs_dict
        )
        return result

    except xmlrpc.client.Fault as e:
        # Handle XML-RPC specific errors (e.g., Odoo operational errors)
        raise Exception(f"Odoo XML-RPC Fault: {e.faultString}") from e
    except ConnectionRefusedError as e:
        raise Exception(
            f"Could not connect to Odoo at {url}. Connection refused."
        ) from e
    except Exception as e:
        # General exception handling
        raise Exception(
            f"An error occurred while communicating with Odoo: {str(e)}"
        ) from e


def get_invoice_addresses(user):
    """Used during organization creation: retrieves all invoice
    addresses the user owns or is connected to from the Odoo API."""
    or_conditions = [
        ("email", "ilike", user.email),
        ("child_ids.email", "ilike", user.email),
    ]

    # Attempt to find the Odoo user ID and add condition for records created by this user
    try:
        odoo_users = odoo_request(
            model="res.users",
            method="search_read",
            domain=[("login", "=", user.email)],
            fields=["id"],
            limit=1,  # Expecting at most one user
        )
        if odoo_users and (uid := odoo_users[0].get("id")):
            or_conditions.append(("create_uid", "=", uid))
    except Exception:
        pass

    user_conditions = ["|"] * (len(or_conditions) - 1) + or_conditions
    # The domain requires the partner to be an invoice address, that is:
    # Of the company_type=person, and type=invoice.
    # If we were searching for an existing organization, we would also have to
    # filter for parent_id=odoo_company_id
    domain = [
        ("company_type", "=", "person"),
        ("type", "=", "invoice"),
    ] + user_conditions

    try:
        invoice_addresses = odoo_request(
            model="res.partner",
            method="search_read",
            domain=domain,
            fields=ADDRESS_FIELDS,
        )
        return invoice_addresses or []
    except Exception:
        return []
